/* Declare draw.cc */



// verify if has been declared, if no do it
#ifndef __3DFC_DRAW_H__



// signals file inclusion
#define __3DFC_DRAW_H__



class CDraw {

public:

   CDraw ();
   CDraw ( CLightTable& oLgT, INT nResX, INT nResY, CMem& oFrBuff );
   ~CDraw ();

protected:

   INT m_nSize;                 // frame buffer size in bytes (resX * resY)
   CMem* m_poFrBuff;            // frame buffer pointer
   CLightTable* m_poLightTable; // light table pointer
   // polygon data
   INT*  m_panForwardX;     // the x's of the polygon edge in forward scan
   INT*  m_panBackwardX;    // the x's of the polygon edge in backward scan
   // texture data
   tagXY* m_pastForwardTex;   // the x's texture in forward poly scan
   tagXY* m_pastBackwardTex;  // the x's texture in backward poly scan
   // needed only if texture has other size than: 64x64 or 256x256
   INT m_nTexWidth;           // texture X dimension
   INT m_nTexHeight;          // texture Y dimension

   void mScanFlatEdge ( INT* panEdgeX, tagXY& stV1, tagXY& stV2 );
   void mScanTexturedEdge ( INT* panEdgePol, tagXY* pastEdgeTex,
                            tagXY& stP1, tagXY& stP2,
                            tagXY& stT1,  tagXY& stT2 );
   void mHorizontalLine ( INT nXIni, INT nXEnd, UINT uCol,
                          PTRVAL pLineAddress );
   void mHorizontalTexturedLine ( INT nXIni, INT nXEnd, tagXY& stTexIni,
                                  tagXY& stTexEnd, BYTE yTypeSize,
                                  tagInterval& stLimits,
                                  BYTE* pyTex, PTRVAL pLineAddress,
                                  DWORD dwLightLevel );

public:

   INT m_nResX;          // frame buffer width
   INT m_nResY;          // frame buffer height
   INT m_nMidX;          // frame buffer midle X value (res X / 2)
   INT m_nMidY;          // frame buffer midle Y value (res Y / 2)
   INT m_nMaxX;          // frame buffer maximum X value (res X - 1)
   INT m_nMaxY;          // frame buffer maximum Y value (res Y - 1)

   // basic functions
   BOOL mCreate ( CLightTable& oLgT, INT nResX, INT nResY, CMem& oFrBuff );
   void mDestroy ();
   void mSetPixel ( INT nX, INT nY, BYTE yCol );
   void mClear ( BYTE yCol );
   void mCopy ( CMem& oFrBuffSource );
   void mCopy ( INT nOX, INT nOY, INT nWidth, INT nHeight,
                CMem& oFrBuffSource );
   void mLine ( INT nXI, INT nYI, INT nXF, INT nYF, BYTE yCol );

   // portal functions (clip)
   void mResetClipEdgeBuffer ( tagInterval* pastEdgeBuffer );
   BOOL mSetClipEdgeBuffer ( tagXY* pastVertexes, INT nNumVertexes,
                             tagInterval* pastEdgeBuffer,
                             tagRectangle& stClipBox );

   // poly draw functions
   void mFlatPolygon4 ( INT nX1, INT nY1, INT nX2, INT nY2,
                        INT nX3, INT nY3, INT nX4, INT nY4, BYTE yCol );
   void mFlatPolygon ( tagXY* pastVertexes, INT nNumVertexes,
                       BYTE yCol );
   void mTexturedPolygon ( tagXY* pastPolVertexes,
                           tagXY* pastTexVertexes,
                           INT nNumVertexes, CTexture& oTx,
                           tagInterval* pastEdgeBuffer = NULL,
                           float fNormLightLevel = 1 );
};



// end declarations
#endif
