/* Store data for a 3D object: polygon mesh, internal sectors

TIPS:
- this class derive from CMesh, see CMesh for more info

WARNING:
- after change object data call CObject::mBeginAddObject and imediatly before
  call CObject::mEndAddObject.
*/



#include <string.h>

#include "defs.h"

#include "node.h"

#include "nodeint.h"

#include "nodepol.h"

#include "list.h"

#include "map.h"

#include "mesh.h"

#include "sector.h"

#include "camsys.h"

#include "nodesec.h"

#include "object.h"



/* constructor - init empty object
**********************************************************/
CObject::CObject ()
{
}



/* destructor - desalocate object data, if alocated
**********************************************************/
CObject::~CObject ()
{
   mDestroy ();
}



/* Desalocate object data, if alocated
**********************************************************/
void CObject::mDestroy ()
{
   m_oSectors.mRemoveAll ();
   m_oRefSys.mDestroy ();

   CMesh::mDestroy ();
}



/* Prepare object to reseve data.
return TRUE - ok / FALSE - error
Call after change data, before call CObject::mEndAddObject
**********************************************************/
BOOL CObject::mBeginAddObject ()
{
   return mBeginAddMesh ();
}



/* Update object data acording definition.
return TRUE - ok / FALSE - error
Call after define all data and before use this data.
**********************************************************/
BOOL CObject::mEndAddObject ()
{
   return mEndAddMesh ();
}



/* Insert a new sector in object. dwSectorID is the sector ID.
To change sector data you must first call CSector::mBeginAddSector.
return a pointer to created sector or NULL if error.
**********************************************************/
CNodeSector* CObject::mAddSector ( DWORD dwSectorID, DWORD dwMyID )
{
   // if key already defined, ID exist, return
   if ( m_oSectors [ dwSectorID ] != NULL ) {
      printf ( MSG_ERROR_SECTOR_ID_ALREADY_DEFINED, dwMyID, dwSectorID );
      return NULL;
   }

   // try create sector node
   CNodeSector
      *poSector = new CNodeSector;
   if ( poSector == NULL ) {
      printf ( MSG_ERROR_MEMORY_LOW );
      printf ( MSG_DEFAULT_ABORT_INFO, __FILE__, __LINE__ );
      return NULL;
   }

   // add in object
   if ( ! m_oSectors.mAddKey ( dwSectorID, (CNode*) poSector ) ) {
      printf ( MSG_ERROR_MEMORY_LOW );
      printf ( MSG_DEFAULT_ABORT_INFO, __FILE__, __LINE__ );
      return NULL;
   }

   return poSector;
}



/* Search in the object sectors for a given object instance and, if
   not found, in the objects that are inside the sectors.
return the object pointer or NULL if not found.
**********************************************************/
CNodeObject* CObject::mSearchObject ( DWORD dwObjID )
{
   CNodeSector
      *poSec;
   CNodeObject
      *poObj;

   // scan sectors vailable
   DWORD
      dwNumSectors = m_oSectors.mRetSize ();
   for ( DWORD dwcInd = 0; dwcInd < dwNumSectors; dwcInd ++ ) {

      // for each key (sector)
      DWORD
         dwKey = m_oSectors.mRetKey ( dwcInd );
      poSec = (CNodeSector*) m_oSectors.mRetValue ( dwKey );

      // verify if found (search return not NULL) else continue search
      if ( ( poObj = (CNodeObject*) poSec->mSearchObject ( dwObjID ) ) != NULL )
         // return the object
         return poObj;         
   }

   // if not found
   return NULL;
}
